@@ -1,3 +1,5 @@
+use crate::node::Axis;
+
 #[derive(Debug, Copy, Clone, PartialEq, Eq, PartialOrd, Ord)]
 pub struct Point {
 	pub x: isize,
@@ -15,6 +17,13 @@ impl Point {
 	pub fn zero() -> Self {
 		Self::new(0, 0)
 	}
+
+	pub fn add_to_axis(&mut self, axis: Axis, operand: isize) {
+		*match axis {
+			Axis::Horizontal => &mut self.x,
+			Axis::Vertical   => &mut self.y,
+		} += operand as isize;
+	}
 }
 
 #[derive(Debug, Copy, Clone, PartialEq, Eq, PartialOrd, Ord)]
@@ -36,6 +45,8 @@ impl Size {
 	}
 }
 
+pub type Spot = (Point, Size);
+
 pub fn aspect_ratio(w: usize, h: usize) -> f64 {
 	(w as f64) / (h as f64)
 }
