@@ -1,16 +1,21 @@
 use crate::app::Application;
-use crate::app::DataRequest;
 use crate::geometry::aspect_ratio;
-use crate::app::rc_widget;
-use crate::app::Widget;
-use crate::tree::LengthPolicy;
-use crate::tree::NodeKey;
+use crate::node::rc_node;
+use crate::node::NodePath;
+use crate::node::Node;
+use crate::node::LengthPolicy;
+use crate::Spot;
 use crate::Point;
 use crate::Size;
 use crate::Void;
+use crate::format;
 
 #[cfg(feature = "xml")]
 use crate::xml::Attribute;
+#[cfg(feature = "xml")]
+use crate::xml::unexpected_attr;
+#[cfg(feature = "xml")]
+use crate::app::DataRequest;
 
 use railway::Program;
 use railway::Address;
@@ -21,7 +26,8 @@ use core::any::Any;
 use std::string::String;
 use std::vec::Vec;
 use std::println;
-use std::format;
+#[cfg(not(feature = "std"))]
+use std::print;
 
 /// See [`xml_handler`]
 #[derive(Debug, Clone)]
@@ -32,6 +38,8 @@ pub struct Railway {
 	pub(crate) size_arg: Address,
 	pub(crate) time_arg: Option<Address>,
 	pub(crate) mask: Vec<u8>,
+	pub(crate) node_size: Size,
+	pub(crate) position: Point,
 	// TODO later: theming
 }
 
@@ -45,7 +53,7 @@ impl Railway {
 		program.valid().ok_or(format!("Invalid railway file"))?;
 		let size_arg = program
 			.argument("size")
-			.ok_or(format!("Missing size in railway file"))?;
+			.ok_or(String::from("Missing size in railway file"))?;
 		let size = stack[size_arg as usize];
 		let ratio = aspect_ratio(size.x as usize, size.y as usize);
 		let time_arg = program.argument("time");
@@ -56,29 +64,44 @@ impl Railway {
 			size_arg,
 			time_arg,
 			mask: Vec::new(),
+			position: Point::zero(),
+			node_size: Size::zero(),
 		})
 	}
 }
 
-impl Widget for Railway {
+impl Node for Railway {
 	fn as_any(&mut self) -> &mut dyn Any {
 		self
 	}
 
-	fn legend(&mut self, _: &mut Application, _: NodeKey) -> String {
+	fn describe(&self) -> String {
 		String::from("Railway file")
 	}
 
-	fn render(&mut self, app: &mut Application, node: NodeKey) -> Void {
-		let (position, size) = app.tree.get_node_spot(node)?;
+	fn policy(&self) -> LengthPolicy {
+		LengthPolicy::AspectRatio(self.ratio)
+	}
+
+	fn get_spot(&self) -> Spot {
+		(self.position, self.node_size)
+	}
+
+	fn set_spot(&mut self, spot: Spot) -> Void {
+		self.position = spot.0;
+		self.node_size = spot.1;
+		None
+	}
+
+	fn render(&mut self, app: &mut Application, _path: &mut NodePath) -> Void {
 		let _ = self.time_arg;
-		self.mask.resize(size.w * size.h, 0);
-		self.stack[self.size_arg as usize] = Couple::new(size.w as f32, size.h as f32);
+		self.mask.resize(self.node_size.w * self.node_size.h, 0);
+		self.stack[self.size_arg as usize] = Couple::new(self.node_size.w as f32, self.node_size.h as f32);
 		self.program.compute(&mut self.stack);
-		let offset = 4 * ((position.y as usize) * app.output.size.w + (position.x as usize));
-		let pitch = 4 * (app.output.size.w - size.w);
+		let offset = 4 * ((self.position.y as usize) * app.output.size.w + (self.position.x as usize));
+		let pitch = 4 * (app.output.size.w - self.node_size.w);
 		let dst = &mut app.output.pixels[offset..];
-		self.program.render(&self.stack, dst, &mut self.mask, size.w, size.h, pitch);
+		self.program.render(&self.stack, dst, &mut self.mask, self.node_size.w, self.node_size.h, pitch);
 		None
 	}
 }
@@ -86,23 +109,21 @@ impl Widget for Railway {
 #[derive(Debug, Copy, Clone)]
 pub struct RailwayLoader;
 
-impl Widget for RailwayLoader {
+impl Node for RailwayLoader {
 	fn as_any(&mut self) -> &mut dyn Any {
 		self
 	}
 
-	fn legend(&mut self, _: &mut Application, _: NodeKey) -> String {
+	fn describe(&self) -> String {
 		String::from("Loading railway file...")
 	}
 
-	fn loaded(&mut self, app: &mut Application, mut node: NodeKey, _: &str, _: usize, data: &[u8]) -> Void {
+	fn loaded(&mut self, app: &mut Application, path: &NodePath, _: &str, _: usize, data: &[u8]) -> Void {
 		let railway = match Railway::new(data) {
 			Err(s) => (println!("{}", s), None).1?,
 			Ok(r) => r,
 		};
-		app.tree.set_node_policy(&mut node, Some(LengthPolicy::AspectRatio(railway.ratio)));
-		app.tree.set_node_widget(&mut node, Some(rc_widget(railway)));
-		app.tree.compute_flexbox(app.tree.get_node_root(node));
+		app.replace_node(path, rc_node(railway)).unwrap();
 		None
 	}
 }
@@ -113,24 +134,23 @@ impl Widget for RailwayLoader {
 /// instance parses the png image and replaces itself with a [`Bitmap`]
 /// containing the decoded image.
 #[cfg(feature = "xml")]
-pub fn xml_handler(app: &mut Application, parent: Option<&mut NodeKey>, attributes: &[Attribute]) -> Result<NodeKey, String> {
+pub fn xml_handler(app: &mut Application, path: &mut NodePath, attributes: &[Attribute]) -> Result<(), String> {
 	let mut source = Err(String::from("missing src attribute"));
 
 	for Attribute { name, value } in attributes {
 		match name.as_str() {
 			"src" => source = Ok(value.clone()),
-			_ => Err(format!("unexpected attribute: {}", name))?,
+			_ => unexpected_attr(&name)?,
 		}
 	}
 
-	let mut node = app.tree.add_node(parent, 3);
-	app.tree.set_node_widget(&mut node, Some(rc_widget(RailwayLoader {})));
-	app.tree.set_node_policy(&mut node, Some(LengthPolicy::AspectRatio(1.0)));
-	app.tree.set_node_spot(&mut node, Some((Point::zero(), Size::zero())));
+	path.push(app.add_node(path, rc_node(RailwayLoader))?);
+
 	app.data_requests.push(DataRequest {
-		node,
+		node: path.clone(),
 		name: source?,
 		range: None,
 	});
-	Ok(node)
+
+	Ok(())
 }
